/*
 *  init.c - ipfm initialisation functions
 *
 *  Copyright (C) 1999 Andres Krapf <dae@via.ecp.fr>
 *
 */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <pcap.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>

#include "utils.h"
#include "pcap.h"
#include "filter.h"
#include "data.h"
#include "init.h"


extern struct s_filter * p_filter;
extern struct s_data * p_data;
extern char * device;
extern char * filemask;
extern unsigned long int next_time;
extern unsigned long int time_interval;
extern int SigHup;

struct OptionsType Options;

void ParseCmdLine(int argc, char * argv[])
{
  char ch;

  Options.ConfigFile = NULL;
  Options.Sort = DEFAULT_OPTIONS_SORT;
  Options.SortFunc = NULL;
  Options.ReverseLookup = DEFAULT_OPTIONS_REVERSELOOKUP;

  while (EOF != (ch = getopt(argc, argv, "c")))
    switch(ch)
    {
    case 'c':
      Options.ConfigFile = strdup(optarg);
      break;
    default:
      /* fprintf(stderr, "wrong option\n"); */
    }
}

void InitOnce()
{
  signal(SIGHUP, SigHupHandler);
  signal(SIGTERM, SigTermHandler);
  signal(SIGKILL, SigKillHandler);
  signal(SIGCHLD, SigChldHandler);

  if (NULL == Options.ConfigFile)
  {
    Options.ConfigFile = strdup(DEFAULT_OPTIONS_CONFIGFILE);
  }
}

void ReInit()
{
  /* Set default values */
  Options.Sort = 0;
  Options.ReverseLookup = 1;

  configure();

  if (time_interval == 0) {
    printf("No time interval specified. Using 1 day as default.\n");
    time_interval = 24*60*60;
  }
  /* Set the time of the next dump */
  next_time = time(NULL) + time_interval;

  if (NULL == filemask) {
    printf("no output file specified. Exiting.\n");
    exit;
  }

  if (NULL == device) {
    char errbuf[PCAP_ERRBUF_SIZE];
    char * tdev;

    tdev = pcap_lookupdev(errbuf);

    if (!tdev)
    {
      fprintf(stderr, "pcap_lookupdev error: %s\n", errbuf);
      exit(1);
    }

    device = strdup(tdev);
  }

  if(!openpcap(device, 1))
  {
    printf("[ipfm] Unable to open pcap descriptor on device %s\n", device);
    /* Exit(); */
    exit(-1);
  }
}

void Init()
{
  InitOnce();
  ReInit();
}

void Clean()
{
  struct s_filter * pTempRule;
  struct s_data * pTempData;

/* clean the rules (deallocate the chained list) */

  while (NULL != p_filter)
  {
    pTempRule = p_filter;
    p_filter = p_filter->next;
    if (pTempRule)
      free(pTempRule);
    else
      fprintf(stderr, "[init.c] BUG!\n");
  }

/* clean the stats (the same) */

  while (NULL != p_data)
  {
    pTempData = p_data;
    p_data = p_data->next;
    if (pTempData)
      free(pTempData);
    else
      fprintf(stderr, "[init.c] BUG 2!\n");
  }

/* close the pcap descriptor */
  closepcap();

  if (filemask)
  {
    free(filemask);
    filemask = NULL;
  }
  if (device)
  {
    free(device);
    device = NULL;
  }
}

void Exit(int errcode)
{
  Clean();

  if (Options.ConfigFile)
    free(Options.ConfigFile);

  exit(errcode);
}

void SigHupHandler(int a)
{
  /*
   * on flushe, on sait jamais. il faudrait faire quelque chose 
   * pour eviter d'ecraser les fichiers au moment du flush
   */

  SigHup = 1;

  printf("IPFM : Received SigHUP\n");
}

void SigTermHandler(int a)
{
  /* 
   * use SIGTERM to flush the data so that the user 
   * can do it any time he wants
   */

  char * filename;

  printf("IPFM : Received SigTERM\n");
  filename = timefile(filemask);
  data_flush(filename);
  free (filename);
}

void SigKillHandler(int a)
{
  printf("IPFM : Received SigKILL\n");
  Exit(0);
}

void SigChldHandler(int a)
{
  int status;
  wait(&status);
}
